"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.deleteFromCache = exports.mock = void 0;

var _path = _interopRequireDefault(require("path"));

var _module = _interopRequireDefault(require("module"));

var _getCallerFile = _interopRequireDefault(require("get-caller-file"));

const _Module = _module.default;
const originalLoad = _Module._load;
const mocked = new Map();

const getMocks = meta => {
  if (mocked.has(meta.filename)) {
    return mocked.get(meta.filename);
  }

  if (meta.parent !== null) {
    return getMocks(meta.parent);
  }

  return null;
};

const isCacheRelated = (target, meta) => {
  if (target === meta.filename) {
    return true;
  }

  if (meta.parent !== null) {
    return isCacheRelated(target, meta.parent);
  }

  return false;
};

const uncache = target => {
  Object.keys(_Module._cache).forEach(key => {
    const meta = _Module._cache[key];

    if (isCacheRelated(target, meta)) {
      Reflect.deleteProperty(_Module._cache, key);
    }
  });
};

const mock = (file, mocks) => {
  const callerDir = _path.default.dirname((0, _getCallerFile.default)());

  const targetPath = _path.default.resolve(callerDir, file);

  const fullPath = require.resolve(targetPath);

  uncache(fullPath);
  mocked.set(fullPath, mocks);

  if (mocked.size === 1) {
    _Module._load = (request, meta, ...rest) => {
      const mocks = getMocks(meta);

      if (mocks !== null && Reflect.has(mocks, request)) {
        const mock = mocks[request];
        Reflect.defineProperty(mock, '__esModule', {
          value: true
        });
        return mock;
      }

      return originalLoad(request, meta, ...rest);
    };
  }

  return () => {
    uncache(fullPath);
    mocked.delete(fullPath);

    if (mocked.size === 0) {
      _Module._load = originalLoad;
    }
  };
};

exports.mock = mock;

const deleteFromCache = target => {
  if (_path.default.isAbsolute(target)) {
    uncache(target);
  } else if (target.startsWith('.')) {
    const callerDir = _path.default.dirname((0, _getCallerFile.default)());

    const targetPath = _path.default.resolve(callerDir, target);

    const fullPath = require.resolve(targetPath);

    uncache(fullPath);
  } else {
    uncache(require.resolve(target));
  }
};

exports.deleteFromCache = deleteFromCache;